//+------------------------------------------------------------------+
//|                                         Copyright 2011, AlexSTAL |
//|                                           http://www.alexstal.ru |
//+------------------------------------------------------------------+
#property copyright "Copyright 2011, AlexSTAL"
#property link      "http://www.alexstal.ru"
#property version   "1.00"

#include <CustPrevCalculated.mqh>
#include <ATRsample.mqh>

#property indicator_separate_window
#property indicator_buffers 1
#property indicator_plots   1
//---- plot Zigzag
#property indicator_type1   DRAW_LINE
#property indicator_color1  DodgerBlue
#property indicator_label1  "ATR"

input uchar InpAtrPeriod = 14; // Period
// Number of bars
input int iBars = 100;

// Buffers of the indicator
double    ExtATRBuffer[];

CCustPrevCalculated CustPrevCalculated;
CCustATR ATR;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
   //--- indicator buffers mapping
   SetIndexBuffer(0, ExtATRBuffer, INDICATOR_DATA);
   ArraySetAsSeries(ExtATRBuffer, true);
   PlotIndexSetDouble(0, PLOT_EMPTY_VALUE, EMPTY_VALUE);
   IndicatorSetInteger(INDICATOR_DIGITS, _Digits);
   
   CustPrevCalculated.InitData(_Symbol, _Period, iBars, CPCHSM_Normal, 0, 30);
   ATR.Init(GetPointer(CustPrevCalculated), _Symbol, _Period, iBars, CPCHSM_Normal, 0, 30, InpAtrPeriod);
   //---
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime& time[],
                const double& open[],
                const double& high[],
                const double& low[],
                const double& close[],
                const long& tick_volume[],
                const long& volume[],
                const int& spread[])
  {
   //---
   int i;
   for (i = ((rates_total-prev_calculated)==0?0:(rates_total-prev_calculated-1)); i >= 0; i--)
     {
      ExtATRBuffer[i] = EMPTY_VALUE;
     }
   CPCPrepareDataResultCode resData = CustPrevCalculated.PrepareData();
   CPCPrepareDataResultCode resATR = ATR.PrepareData(resData);
   if ( (resATR != CPCPDRC_NoData) && (resATR != CPCPDRC_NoRecountNotRequired) )
      ATR.Calculate();
   
   PlotIndexSetInteger(0, PLOT_DRAW_BEGIN, rates_total - ATR.GetBarsCount() + InpAtrPeriod);
   
   for (i = 0; i < (int)MathMin(ATR.GetBarsCount(), rates_total); i++)
     {
      ExtATRBuffer[i] = ATR.GetATR(i, true);
     }
   //--- return value of prev_calculated for next call
   return(rates_total);
  }
//+------------------------------------------------------------------+
